﻿using NetBootServerCore.Components.Logging;
using System.Text;

namespace NetBootServerCore.Components.Http.StaticContent
{
    public class StaticContentProcessor : NbsComponentBase, IHttpRequestProcessor
    {
        private readonly IConfigurationSection _config;
        private readonly Dictionary<string, byte[]> _contentMap = new();

        private string _basePath;
        private string _contentPath;
        private string _defaultDocument;
        private bool _enableFallbackRoute;
        private string _htmlBaseReplace;


        private bool _isRunning;

        public StaticContentProcessor(IConfigurationSection config) : base(new Logger("HTTP.Static"))
        {
            _config = config;
        }

        protected override void OnInit()
        {
            _basePath = _config["Base"].TrimEnd('/');
            _contentPath = _config["Path"];
            _defaultDocument = _config["DefaultDocument"];
            _enableFallbackRoute = bool.Parse(_config["EnableFallbackRoute"]);
            _htmlBaseReplace = _config["HtmlBaseReplace"];
        }

        protected override void OnStart()
        {
            if (File.Exists(_contentPath))
            {
                using var arc = SharpCompress.Archives.ArchiveFactory.Open(_contentPath);
                foreach (var ent in arc.Entries.Where(p => p.IsDirectory == false))
                {
                    var buf = new byte[ent.Size];
                    using var s = ent.OpenEntryStream();
                    var r = s.Read(buf, 0, buf.Length);
                    _contentMap[ent.Key.ToLower()] = buf;
                }
            }
            else if (Directory.Exists(_contentPath))
            {
                var files = Directory.GetFiles(_contentPath, "*", System.IO.SearchOption.AllDirectories);
                foreach (var item in files)
                {
                    var k = item.Substring(_contentPath.Length + 1).Replace("\\", "/").ToLower();
                    _contentMap[k] = File.ReadAllBytes(item);
                }
            }
            else throw new IOException("Content path not found");

            _isRunning = true;
        }

        protected override void OnStop()
        {
            _isRunning = false;
            _contentMap.Clear();
        }

        public async Task<bool> ProcessRequest(HttpContext context)
        {
            if (!_isRunning) return false;

            var requestPath = context.Request.Path;

            if (requestPath == "/")
            {
                context.Response.Redirect($"{_basePath}{_defaultDocument}");
                return true;
            }

            if (requestPath.StartsWithSegments(_basePath))
            {
                var arcPath = ((string)requestPath)[(_basePath.Length + 1)..].ToLower();

                if (_enableFallbackRoute && false == _contentMap.ContainsKey(arcPath))
                {
                    arcPath = _defaultDocument;
                }

                if (false == _contentMap.TryGetValue(arcPath, out var bin))
                {
                    context.Response.StatusCode = 404;
                    return true;
                }

                if (arcPath.ToLower().EndsWith(".js")) context.Response.ContentType = "application/javascript";
                else if (arcPath.ToLower().EndsWith(".wasm")) context.Response.ContentType = "application/wasm";
                else if (arcPath.ToLower().EndsWith(".css")) context.Response.ContentType = "text/css";
                else if (arcPath.ToLower().EndsWith(".html"))
                {
                    context.Response.ContentType = "text/html";
                    if (_htmlBaseReplace != null)
                    {
                        var html = Encoding.UTF8.GetString(bin);
                        var r = html.Replace(_htmlBaseReplace, $"<base href=\"{_basePath}/\" />");
                        bin = Encoding.UTF8.GetBytes(r);
                    }
                }
                else context.Response.ContentType = "application/octet-stream";

                context.Response.ContentLength = bin.Length;
                await context.Response.BodyWriter.WriteAsync(bin);
                return true;
            }

            return false;
        }
    }
}
