using NetBootServerCore.Components.Http;
using NetBootServerCore.Components.Http.StaticContent;
using NetBootServerCore.Components.Http.WebApi;
using NetBootServerCore.Components.Logging;
using NetBootServerCore.Components.NetBoot;
using NetBootServerCore.Components.NetBoot.Dhcp;
using NetBootServerCore.LivingConsole;

namespace NetBootServerCore;

public static class NbsProgram
{
    public static readonly DateTime LaunchTime;

    static NbsProgram()
    {
        LaunchTime = DateTime.Now;
    }

    private static void Main(string[] args)
    {
        LiveConsole.Start();

        var logger = new Logger("APP");

        logger.LogInfo($"Launch Time: {LaunchTime:yyyy-MM-dd HH:mm:ss.fff}");

        logger.LogDebug("Logger Init");
        logger.LogVerbose("Logger Init");
        logger.LogInfo("Logger Init");
        logger.LogWarn("Logger Init");
        logger.LogError("Logger Init");
        logger.LogFatal("Logger Init");

        logger.LogInfo("Load config");
        var config = WebApplication.CreateBuilder(args).Configuration;
        var httpPrefix = config.GetSection("Web")["Prefix"];
        Console.Title = $"{nameof(NetBootServerCore)} at {httpPrefix}";
        Environment.SetEnvironmentVariable("ASPNETCORE_URLS", httpPrefix);

        logger.LogInfo("Create components");
        var dhcp = new DhcpServer(config.GetSection("DHCP"));
        var tftp = new TftpServer(config.GetSection("TFTP"));
        var bsp = new BootScriptProvider(config.GetSection("BootScript"));

        var scriptModule = new BootScriptProcessor(bsp);
        var apiModule = new WebApiProcessor(dhcp);
        var staticModule = new StaticContentProcessor(config.GetSection("StaticContent"));
        var httpServer = new HttpServer(args,new IHttpRequestProcessor[] { scriptModule, apiModule, staticModule });

        logger.LogInfo("Init components");
        dhcp.Init();
        tftp.Init();
        scriptModule.Init();
        apiModule.Init();
        staticModule.Init();
        httpServer.Init();

        logger.LogInfo("Start components");
        dhcp.Start();
        tftp.Start();
        scriptModule.Start();
        apiModule.Start();
        staticModule.Start();
        httpServer.Start();

        logger.LogInfo("Running");

        //Waiting until Control-C
        {
            CancellationTokenSource ctSource = new();
            var ct = ctSource.Token;

            void ExitHandler()
            {
                // You can add any arbitrary global clean up
                logger.LogInfo("Exiting...");
                ctSource.Cancel();
            }

            // Assign exit handler to be called when the process is terminated
            // or the user hits CTRL+C
            AppDomain.CurrentDomain.ProcessExit += (_, _) => ExitHandler();
            Console.CancelKeyPress += (_, _) => ExitHandler();

            // Then you can use the cancellation token to check for exit:
            while (!ct.IsCancellationRequested)
            {
                try
                {
                    Task.Delay(1000, ct).Wait(ct);
                }
                catch (OperationCanceledException e)
                {
                    break;
                }
            }
        }

        //stop components
        dhcp.Stop();
        tftp.Stop();
        apiModule.Stop();
        scriptModule.Stop();
        staticModule.Stop();
        httpServer.Stop();

        logger.LogInfo("Finished");

        LiveConsole.Stop();
    }
}
